package org.usfirst.frc.team000.robot.commands;

import org.usfirst.frc.team000.robot.Robot;

import edu.wpi.first.wpilibj.command.Command;

//import java.util.concurrent.TimeUnit;

/**
 *
 */
public class Auto extends Command {

	enum Action {
		enable_wrist,
		enable_elevator,
		open_claw,
		wrist_up,
		elevator_down,
		close_claw,
		elevator_up,
		reset_drive,
		drive_forward,
		wrist_down,
		drive_back,
		stop,
		done
	};
	
	Action [] actions;
		
//	boolean	done = false;
	int action;
	long start_time;
	
    public Auto() {
        // Use requires() here to declare subsystem dependencies
        requires(Robot.drivetrain);
        requires(Robot.claw);
    }

    // Called just before this Command runs the first time
    protected void initialize() {
    	action = 0;
//    	done = false;
    	start_time = System.currentTimeMillis();
    	actions = new Action [] {
       		Action.reset_drive,
       		Action.open_claw,    			
    		Action.enable_elevator,
    		Action.elevator_down,
    		Action.close_claw,
    		Action.enable_wrist,
    		Action.wrist_up,
    		Action.enable_elevator,
    		Action.elevator_up,
    		Action.drive_forward,
    		Action.stop,
    		Action.enable_wrist,
    		Action.wrist_down,
    		Action.open_claw,
    		Action.enable_wrist,
    		Action.wrist_up,
    		Action.drive_back,
    		Action.stop,
    		Action.done
    	};
    }
    
    // Called repeatedly when this Command is scheduled to run
    protected void execute() {
    	if (action>=actions.length) return;
    	System.out.println("Next action "+action+": "+actions[action]);
		switch (actions[action]) {
		case open_claw:
			open_claw();
			break;
		case enable_wrist:
			enable_wrist();
			break;
		case wrist_up:
			move_wrist(-45);
			break;
		case enable_elevator:
			enable_elevator();
			break;
		case elevator_down:
			put_down();
			break;
		case close_claw:
			close_claw();
			break;
		case elevator_up:
			lift_up();
			break;
		case reset_drive:
			reset_drive();
			break;
		case drive_forward:
			drive(1.6, 0.2); 
			break;
		case wrist_down:
			move_wrist(0.0);
			break;
		case drive_back:
			drive(1.8, -0.2);
			break;
		case stop:
			stop();
		case done:
			done();
		default:
		}
    }

    // Make this return true when this Command no longer needs to run execute()
    protected boolean isFinished() {
//?    	if (action == actions.length) return true;
        return false;
    }

    // Called once after isFinished returns true
    protected void end() {
    }

    // Called when another command which requires one or more of the same
    // subsystems is scheduled to run
    protected void interrupted() {
    }


    protected void next() {
    	action++;
    	pause();
    }
    
    protected void enable_wrist() {
    	if (is_waiting()) return;
    	Robot.wrist.enable(); 
    	next(); 
    }
    
    protected void move_wrist(double angle) {
    	if (is_waiting()) return;
    	Robot.wrist.setSetpoint(angle);
    	next();
    }
    
    protected void open_claw() {
    	if (is_waiting()) return;
    	Robot.claw.open(); 
    	next();
    }

    protected void close_claw() {
    	if (is_waiting()) return;
    	Robot.claw.close(); 
    	next();
    }

    protected void enable_elevator() {
    	if (is_waiting()) return;
    	Robot.elevator.enable(); 
    	next();
    }

    protected void lift_up() {
    	if (is_waiting()) return;
    	Robot.elevator.setSetpoint(0.22);
    	next();
    }

    protected void put_down() {
    	if (is_waiting()) return;
   		Robot.elevator.setSetpoint(0.0);
    	next();
    }

    protected void drive(double distance, double velocity) {
    	final double creeping_speed = 0.0001;
    	if (is_waiting()) return;
		Robot.drivetrain.drive(velocity, velocity);
    	action++;
    	double speed = Math.abs(velocity);
    	if (speed > creeping_speed) 
    		wait(distance/speed);
    	else
    		pause();
    }

    protected void reset_drive() {
    	if (is_waiting()) return;
		Robot.drivetrain.reset();
		next();
    }

    protected void stop() {
    	if (is_waiting()) return;
    	Robot.drivetrain.drive(0.0,0.0);
    	next();
    }
    protected void done() {
    	if (is_waiting()) return;
		Robot.drivetrain.stop();
		next();
    }

    protected void pause() {
    	wait(1.0);
    }

    protected void wait(double delay) {
    	start_time = System.currentTimeMillis() + (long)delay*1000;
    }

    protected boolean is_waiting() {
    	if (System.currentTimeMillis() < start_time)
    		return true;
    	return false;
    }
    
}
